/*	$NetBSD: include.h,v 1.4 1997/10/13 02:18:19 lukem Exp $	*/

/*
 * include.h - includes all important files for Phantasia
 */

#include <ctype.h>
#include <curses.h>
#include <errno.h>
#include <math.h>
#include <setjmp.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "macros.h"
#include "phantdefs.h"
#include "phantstruct.h"
#include "phantglobs.h"
#include "pathnames.h"
