/*	$NetBSD: nightobjs.c,v 1.7 1999/02/10 01:36:50 hubertf Exp $	*/

/*
 * Copyright (c) 1983, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>
#ifndef lint
#if 0
static char sccsid[] = "@(#)nightobjs.c	8.2 (Berkeley) 4/28/95";
#else
__RCSID("$NetBSD: nightobjs.c,v 1.7 1999/02/10 01:36:50 hubertf Exp $");
#endif
#endif				/* not lint */

#include "extern.h"

const struct objs nightobjs[] = {
	{218, PAJAMAS},
	{235, NATIVE},
	{92, PAPAYAS},
	{92, PINEAPPLE},
	{92, KIWI},
	{92, MANGO},
	{92, NATIVE},
	{92, MAN},
	{181, LAMPON},
	{236, LAMPON},
	{92, LAMPON},
	{216, WOODSMAN},
	{216, DEADWOOD},
	{216, MALLET},
	{168, WOODSMAN},
	{168, DEADWOOD},
	{168, MALLET},
	{170, WOODSMAN},
	{170, DEADWOOD},
	{170, MALLET},
	{124, SHIELD},
	{124, HALBERD},
	{124, ELF},
	{144, SHIELD},
	{144, HALBERD},
	{144, ELF},
	{113, SHIELD},
	{113, HALBERD},
	{113, ELF},
	{161, SHIELD},
	{161, HALBERD},
	{161, ELF},
	{169, SHIELD},
	{169, HALBERD},
	{169, ELF},
	{182, SHIELD},
	{182, HALBERD},
	{182, ELF},
	{198, SHIELD},
	{198, HALBERD},
	{198, ELF},
	{212, SHIELD},
	{212, HALBERD},
	{212, ELF},
	{216, SHIELD},
	{216, HALBERD},
	{216, ELF},
	{226, SHIELD},
	{226, HALBERD},
	{226, ELF},
	{228, SHIELD},
	{228, HALBERD},
	{228, ELF},
	{68, CYLON},
	{144, SHOVEL},
	{249, FOOT},
	{250, FOOT},
	{93, PAPAYAS},
	{0, 0}
};
